# -*- coding: utf-8 -*-
"""
    File and filesystem operations module
"""
import os
import shutil
import unicodedata
import codecs
from datetime import datetime
import simplejson as json

class SimpleFileOps(object):
    """Class for easing typical file and filesystem operations"""
    def __init__(self, _Tool):
        """Class initialization procedure

        Args:
            self: The reserved object 'self'
            Tool: The GISTools10 object for data processing
        """
        self.Tool = _Tool

    def ClearDir(self, DirName, searchPatern='*'): # Overriding
        """Directory cleaning automation procedure

        Args:
            self: The reserved object 'self'
            DirName: The directory to be cleaned
        """
        self.Tool.AddMessage(u"\n----------- Cleaning directory [" +  DirName + u"] searching: {" + searchPatern + "} -------" + self.Tool.MyNow())
        self.Tool.RunPS("ClearDir", '"' + self.Tool.CorrectStr(DirName) + '" ' + searchPatern, os.path.dirname(__file__))

    def delFileIfExists(self, fileName): # Overriding
        """"Deletes file if file exists
            
            Args:
                self: The reserved object 'self'
                DirName: The directory to be cleaned

        """
        if os.path.exists(fileName):
            os.remove(fileName)

    def DelClearDir(self, DirName, searchPatern='*'): # Overriding
        """Delete non-empty directory

        Args:
            self: The reserved object 'self'
            DirName: The directory to be deleted
        """
        self.Tool.AddMessage(u"\n----------- Cleaning and deleting directory [" +  DirName + "] -------" + self.Tool.MyNow())
        self.Tool.RunPS("ClearDir", '"' + self.Tool.CorrectStr(DirName) + '" ' + searchPatern, os.path.dirname(__file__))
        os.rmdir(DirName)
        self.Tool.AddMessage(u"    ... directory [" +  DirName + u"] deleted ")

    def BackupFiles(self, InDirName, OutDirName, D): # Overriding
        """File archiving automation procedure (Overriding)

        Args:
            self: The reserved object 'self'
            InDirName: Input directory
            OutDirName: Output directory
            D: How old files to archive (number of days)
        """
        self.Tool.RunPS("BackupOldFiles", '"' + self.Tool.CorrectStr(InDirName) + '" "' + self.Tool.CorrectStr(OutDirName) + '" ' + str(D), os.path.dirname(__file__))

    def GetLog(self, server, EventLogOutputDir, D): # NoneOverriding
        """File archiving automation procedure (None overriding)

        Args:
            self: The reserved object 'self'
            server: Server which eventlog backup to create
            EventLogOutputDir: Event log output directory
            D: How old files to archive (number of days)
        """
        self.Tool.RunPS("GetLog", '"' + server + '" "' + self.Tool.CorrectStr(EventLogOutputDir) + '" '+ str(D), os.path.dirname(__file__))

    def BackupOneFile(self, InFileName, OutDirName): # Overriding
        """Specific file archiving automation procedure

        Args:
            self: The reserved object 'self'
            InFileName: Input file
            OutDirName: Output directory
        """
        self.Tool.AddMessage("")
        self.Tool.AddMessage("--- Archiving file " +  InFileName + " to " + OutDirName + " - " + self.Tool.MyNow())
        if os.path.exists(InFileName):
            shutil.move(InFileName, OutDirName + '\\' + os.path.split(InFileName)[1].split(".")[-2] + "_" + self.Tool.MyNowFile() + os.path.splitext(InFileName)[1])
        else:
            self.Tool.AddMessage("    File " + InFileName + ' does not exist ...')

    def CheckCreateDir(self, OutDirName):
        """Automation procedure which creates directory, in case it doesn't exist

        Args:
            self: The reserved object 'self'
            OutDirName: Output directory
        """
        if not os.path.exists(OutDirName):
            os.makedirs(OutDirName)

    def CheckCreateClearDir(self, DirName):
        """Automation procedure which creates directory, in case it doesn't exist and if it exists then clear this dir

        Args:
            self: The reserved object 'self'
            DirName: Output directory
        """
        self.CheckCreateDir(DirName)
        self.ClearDir(DirName)

    def GetSafeName(self, text, substituteChar='_', aditionalScaryChars='', aditionalSpaceChars='', noDotsInName=False):
        """Modifies the text for use in the filesystem

        Args:
            self: The reserved object 'self'
            text: Text string which will be transformed
            substituteChar: Character to substitute the whitespace
            aditionalScaryChars: Additional characters to eliminate
            aditionalSpaceChars: Additional characters to replace
            noDotsInName: Forbid dots in filename (except the file extension seperator) (Default = False)

        Returns:
            * Modified text as string
        """
        if type(text) is unicode:
            text = unicodedata.normalize('NFD', text).encode('ascii', 'ignore')
        checkChars = ''.join(set(r'[]/\;,><&*:%=+@!#^()|?^-' + aditionalScaryChars))
        for c in checkChars:
            text = text.replace(c, '')
        checkSpaceChars = ''.join(set(' ' + aditionalSpaceChars))
        for c in checkSpaceChars:
            text = text.replace(c, substituteChar)
        if noDotsInName:
            lastDot = text.rfind('.')
            if lastDot != -1:
                text = (text[:lastDot]).replace('.', substituteChar) + text[lastDot:]
        while text.find(substituteChar+substituteChar) > -1:
            text = text.replace(substituteChar+substituteChar, substituteChar)
        return text

    def FindNewestFile(self, Dir, Ext='*'):
        """Finds the newest file in the directory

        Args:
            self: The reserved object 'self'
            Dir: The directory in which to look for the file
            Ext: The extension to search for ('*' - search any file)
        """
        if not Ext == '*':
            dated_files = [(os.path.getmtime(Dir + "\\" + fn), os.path.basename(Dir + "\\" + fn))
                           for fn in os.listdir(Dir)
                          ]
        else:
            dated_files = [(os.path.getmtime(Dir + "\\" + fn), os.path.basename(Dir + "\\" + fn))
                           for fn in os.listdir(Dir)
                           if fn.lower().endswith('.' + (Ext.lower()))
                          ]
        dated_files.sort()
        dated_files.reverse()
        newest = dated_files[0][1]
        return Dir + "\\" + newest

    def FindFileByDate(self, Dir, Ext='*', Date=datetime.now(), Mode='New'):
        """Find files in the given directory which are newer than the given date

        Args:
            self: The reserved object 'self'
            Dir: The directory in which to look for the file
            Ext: The extension to search for ('*' - search any file)
            Date: Find files newer than given date
            Mode: File searching modes: New - search newer files; Old - search older files (Default = New)
        """
        if Ext == '*':
            dated_files = [Dir + "\\" + fn
                           for fn in os.listdir(Dir)
                           if ((Mode == 'New' and datetime.fromtimestamp(os.path.getmtime(Dir + "\\" + fn)) >= Date) or
                               (Mode == 'Old' and datetime.fromtimestamp(os.path.getmtime(Dir + "\\" + fn)) <= Date))
                          ]
        else:
            dated_files = [Dir + "\\" + fn
                           for fn in os.listdir(Dir)
                           if ((Mode == 'New' and datetime.fromtimestamp(os.path.getmtime(Dir + "\\" + fn)) >= Date) or
                               (Mode == 'Old' and datetime.fromtimestamp(os.path.getmtime(Dir + "\\" + fn)) <= Date))
                           and fn.lower().endswith('.' + (Ext.lower()))
                          ]
        dated_files.sort()
        return dated_files

    def FindFile(self, Dir, Ext):
        """Finds files in the directory

        Args:
            self: The reserved object 'self'
            Dir: The directory in which to look for the file
            Ext: The extension to search for
        """
        if Ext == '*':
            files = [os.path.join(Dir,fn)
                        for fn in os.listdir(Dir) if os.path.isfile(os.path.join(Dir,fn))]
        else:
            files = [os.path.join(Dir,fn)
                        for fn in os.listdir(Dir) if fn.lower().endswith('.' + (Ext.lower()))]
        files.sort()
        return files

    def FindDirectory(self, Dir, Searchpattern='*'):
        """Find subdirectories in the given directory

        Args:
            self: The reserved object 'self'
            Dir: The directory in which to look for subdirectories
            Searchpattern: Searching condition
        """
        if Searchpattern == '*':
            dirs = [fn for fn in os.listdir(Dir) if os.path.isdir(os.path.join(Dir,fn))]
        else:
            dirs = [fn for fn in os.listdir(Dir) if os.path.isdir(os.path.join(Dir,fn))
                    and not fn.lower().find(Searchpattern.lower()) == -1]
        dirs.sort()
        return dirs

    def FindFileRecursive(self, Dir, Ext):
        """Find files by extension

        Args:
            self: The reserved object 'self'
            Dir: The directory in which to look for the file
            Ext: The extension to search for
        """
        dated_files = []
        for root, dirnames, filenames in os.walk(Dir):
            for filename in filenames:
                if filename.lower().endswith('.' + (Ext.lower())):
                    dated_files.append(os.path.join(root, filename))
        dated_files.sort()
        return dated_files

    def CopyAllFilesInDir(self, SourceDir, DestDir, Searchpattern='*', Ignorepattern=None):
        """Copy entire directory tree from one directory to another

        Args:
            self: The reserved object 'self'
            SourceDir: Source directory
            DestDir: Destination directory
            Searchpattern: Searching condition
        """
        if not os.path.exists(DestDir):
            os.makedirs(DestDir)

        if Searchpattern == '*' and Ignorepattern==None:
            foundfiles = [fn for fn in os.listdir(SourceDir)]
        elif Searchpattern != '*' and Ignorepattern==None:
            foundfiles = [fn for fn in os.listdir(SourceDir) if not fn.lower().find(Searchpattern.lower()) == -1]
        elif Searchpattern == '*' and Ignorepattern!=None:
            foundfiles = [fn for fn in os.listdir(SourceDir) if fn.lower().find(Ignorepattern.lower()) == -1]
        elif Searchpattern != '*' and Ignorepattern!=None:
            foundfiles = [fn for fn in os.listdir(SourceDir) if not fn.lower().find(Searchpattern.lower()) == -1 and fn.lower().find(Ignorepattern.lower()) == -1]
        foundfiles.sort()

        for item in foundfiles:
            s = os.path.join(SourceDir, item)
            d = os.path.join(DestDir, item)
            if os.path.isdir(s):
                self.CopyAllFilesInDir(s, d, Searchpattern, None)
            else:
                shutil.copy2(s, d)

    def printFile(self, filePath):
        """Print file content to the screen

        Args:
            self: The reserved object 'self'
            filePath: File to print
        """
        with codecs.open(filePath, 'r', 'utf8') as fin:
            self.Tool.AddMessage(fin.read())

    def GetfileNameWithDate(self, file):
        """Add a date at the end of the filename

        Args:
            self: The reserved object 'self'
            file: Full path to the file to add the date
        """
        dirname = os.path.dirname(file)
        filename = ('.').join(file.split('.')[:-1])
        fileext = ('.').join(file.split('.')[-1:])

        return os.path.join(dirname, filename + '_' + self.Tool.MyNowFileSafe() + '.' + fileext)

    def CopareFileLists(self, SourceFiles, DestFiles):
        """Compare two file lists to determine changes
        
        Args:
            SourceFiles (List): Sorce file path list
            DestFiles (List): Destination file path list

        Returns:
            Dictionary:
                AbsentSource (List): Files in Source and not in Destination
                AbsentDestination (List): Files in Destination and not in Source
        """
        AbsentSource = list()
        AbsentDestination = list()

        SourceFileDict = dict()
        for sFile in SourceFiles:
            basename = os.path.basename(sFile)
            ext = os.path.splitext(basename)[1]
            modifyied = datetime.fromtimestamp(os.path.getmtime(sFile))
            SourceFileDict[basename] = {"path": sFile, "ext": ext, "modifyied": modifyied}
            
        DestFileDict = dict()
        for dFile in DestFiles:
            basename = os.path.basename(dFile)
            ext = os.path.splitext(basename)[1]
            modifyied = datetime.fromtimestamp(os.path.getmtime(dFile))
            DestFileDict[basename] = {"path": sFile, "ext": ext, "modifyied": modifyied}

        for sFile in SourceFileDict.keys():
            if not DestFileDict.has_key(sFile):
                AbsentDestination.append(SourceFileDict[sFile]["path"])
            else:
                # print SourceFileDict[sFile]["modifyied"]
                # print DestFileDict[sFile]["modifyied"]
                if SourceFileDict[sFile]["modifyied"].replace(microsecond=0)>DestFileDict[sFile]["modifyied"].replace(microsecond=0):
                    AbsentDestination.append(SourceFileDict[sFile]["path"])

        for dFile in DestFileDict.keys():
            if not SourceFileDict.has_key(dFile):
                AbsentSource.append(DestFileDict[dFile]["path"])

        return {"AbsentSource":AbsentSource, "AbsentDestination":AbsentDestination}

class LockSubprocess(object):
    """Class that provides directory locking control and processing"""
    def __init__(self, Tool, Dir, processName):
        """Class initialization procedure
        Args:
            self: The reserved object 'self'
            Tool: Reference to the GISTools10 object
            Dir: Working directory
            processName: Name of the process
        """
        self.StartTime = datetime.now()
        self.Tool = Tool
        self.Dir = Dir
        self.processName = processName
        self.results = LockResults(processName)
        self.lockFileName = os.path.join(self.Dir, 'gispython.{0}.lock'.format(self.processName))

    def __enter__(self):
        """With statement opening procedure
        Args:
            self: The reserved object 'self'
        Returns:
            LockResults with status:
                Locked - The file is locked by another process;
                DoneBefore -  Process is already done;
                NoDir - Directory not found;
                Running - Process is running;
        """
        # Check if directory exists
        if os.path.exists(self.Dir):
            if os.path.exists(self.lockFileName) and os.path.isfile(self.lockFileName):
                try:
                    self.f = codecs.open(self.lockFileName, 'r+', 'utf-8')
                    self.readJson()
                except:
                    self.results.status = "Locked"
                    return self.results
                if self.J['status'] == 'Running':
                    self.results.status = "Locked"
                    return self.results
                if self.J['status'] == 'Done':
                    self.results.status = "DoneBefore"
                    return self.results
                else:
                    self.results.status = "Running"
                    self.writeJson()
            else:
                self.results.status = "Running"
                self.writeJson()
        else:
            self.results.status = "NoDir"
        return self.results

    def __exit__(self, type, value, traceback):
        """With statement closing procedure
        Args:
            self: The reserved object 'self'
        """
        if self.results.status != "Locked":
            self.results.status = "Done"
            self.writeJson()
        if hasattr(self, 'f'):
            self.f.close()

    def readJson(self):
        """Get the data from the lock file
        Args:
            self: The reserved object 'self'
        """

        JsonString = self.f.read()
        try:
            self.J = json.loads(JsonString)
        except:
            self.J = {'status':'Ok'}

    def writeJson(self):
        """Save parameters in the file
        Args:
            self: The reserved object 'self'
        """
        rezult = {'status': self.results.status,
                  'processName':self.results.processName,
                  'processdate': str(self.results.processdate)}
        JsonString = json.dumps(rezult, sort_keys=True, indent=4 * ' ')
        if hasattr(self, 'f'):
            self.f.close()
        self.f = codecs.open(self.lockFileName, 'w', 'utf-8')
        self.f.truncate()
        self.f.write(JsonString)
        self.f.flush()

class LockResults(object):
    """Class for saving the data processing results (for LockSubprocess)"""

    def __init__(self, processName):
        """Class initialization procedure

        Args:
            self: The reserved object 'self'
            processName: Name of process
        """
        self.status = "Running"
        self.processdate = datetime.now()
        self.processName = processName

    def GetStatusTxt(self):
        """Get status description

        Args:
            self: The reserved object 'self'
        """
        if self.status == "Running":
            return u'Process execution is running'
        if self.status == "Locked":
            return u'This process cannot be continued, because another process is already running'
        if self.status == "DoneBefore":
            return u'Process is already done'
        if self.status == "NoDir":
            return u'Corresponding directory not found'



# -*- coding: utf-8 -*-
"""
    Carries out service caching on ArcServer providing x3 retry capabilities
"""

#Modula pamata klase
class CachingHelper:
    """Support class which generates the caches"""
    def __init__(self, Tool, vServer, vExtent="#", vTerritoryLayer="#"):
        """Class initialization procedure

        Args:
            self: The reserved object 'self'
            Tool: GEOPython tool
            vServer: Server
            vExtent: Extent
            vTerritoryLayer: Territory layer
        """
        self.Tool = Tool
        self.Server = vServer
        self.Extent = vExtent
        self.TerritoryLayer = vTerritoryLayer

    def GenerateCache(self, vService, vInstances, vCashScales, vFolder="#", vDeleteScales='#'):
        """Base procedure of the tool

        Args:
            self: The reserved object 'self'
            vService: Cacheable serviss
            vInstances: Cacheable instance count
            vCashScales: Cacheable scales
            vFolder: Cache service directory
            vDeleteScales: Scales to delete
        """

        if vFolder == "#":
            vFolder = ""
        elif vFolder == None:
            vFolder = ""
        else:
            vFolder = str(vFolder) + "\\"
        vFolder = "\\" +  vFolder

        if vDeleteScales == '#':
            vDeleteScales = vCashScales

        input_service = self.Server + vFolder + str(vService) + '.MapServer'
        if vDeleteScales != "None":
            try: # 1 try to delete
                self.Tool.gp.ManageMapServerCacheTiles_server(input_service, vCashScales, "DELETE_TILES", vInstances, self.TerritoryLayer, self.Extent, 'WAIT')
                self.Tool.OutputMessages()
            except: # 2 try to delete
                self.Tool.OutputMessages()
                self.Tool.AddMessage("Delete Restart x1 ----------------------------------------------- " + self.Tool.MyNow())
                try:
                    self.Tool.gp.ManageMapServerCacheTiles_server(input_service, vCashScales, "DELETE_TILES", vInstances, self.TerritoryLayer, self.Extent, 'WAIT')
                    self.Tool.OutputMessages()
                except: # 3 try to delete
                    self.Tool.OutputMessages()
                    self.Tool.AddMessage("Delete Restart x2 ----------------------------------------------- " + self.Tool.MyNow())
                    try:
                        self.Tool.gp.ManageMapServerCacheTiles_server(input_service, vCashScales, "DELETE_TILES", vInstances, self.TerritoryLayer, self.Extent, 'WAIT')
                        self.Tool.OutputMessages()
                    except:
                        self.Tool.OutputMessages()
                        self.Tool.AddWarning("Delete Restart x3 for service {0} - Failed ------------------------------------- {1} ".format(input_service, self.Tool.MyNow()))

        try: # 1 try
            self.Tool.gp.ManageMapServerCacheTiles_server(input_service, vCashScales, "RECREATE_ALL_TILES", vInstances, self.TerritoryLayer, self.Extent, 'WAIT')
            self.Tool.OutputMessages()
        except: # 2 try
            self.Tool.OutputMessages()
            self.Tool.AddMessage("Restart x1 ----------------------------------------------- " + self.Tool.MyNow())
            try: 
                self.Tool.gp.ManageMapServerCacheTiles_server(input_service, vCashScales, "RECREATE_EMPTY_TILES", vInstances, self.TerritoryLayer, self.Extent, 'WAIT')
                self.Tool.OutputMessages()
            except: # 3 try
                self.Tool.OutputMessages()
                self.Tool.AddMessage("Restart x2 ----------------------------------------------- " + self.Tool.MyNow())
                try:
                    self.Tool.gp.ManageMapServerCacheTiles_server(input_service, vCashScales, "RECREATE_EMPTY_TILES", vInstances, self.TerritoryLayer, self.Extent, 'WAIT')
                    self.Tool.OutputMessages()
                except:
                    self.Tool.OutputMessages()
                    self.Tool.AddWarning("Caching Restart x3 for service {0} - Failed ------------------------------------- {1} ".format(input_service, self.Tool.MyNow()))



# -*- coding: utf-8 -*-
"""
    File and filesystem operations module
"""
import os
import shutil
from datetime import date, datetime, timedelta
import stat
import fnmatch
import SimpleFileOps

class SimpleFileOpsSafe(SimpleFileOps.SimpleFileOps):
    """Class for easing the most typical file and filesystem operations"""
    def __init__(self, _Tool):
        """Class initialization procedure

        Args:
            self: The reserved object 'self'
            Tool: The 'GISTools10' object for data processing
        """
        super(SimpleFileOpsSafe, self).__init__(_Tool)

    def ClearDir(self, DirName, searchPatern = '*'): # Overriding
        """Directory cleaning automation procedure

        Args:
            self: The reserved object 'self'
            DirName: The directory to be cleaned
        """
        self.Tool.AddMessage(u"\n----------- Cleaning directory [" +  DirName + "] -------" + self.Tool.MyNow())
        self._clear_dir(DirName, searchPatern)
        self.Tool.AddMessage(u"    ... directory [" +  DirName + u"] cleaned ")

    def DelClearDir(self, DirName): # Overriding
        """Delete non-empty directory

        Args:
            self: The reserved object 'self'
            DirName: The directory to be deleted
        """
        self.Tool.AddMessage(u"\n----------- Cleaning and deleting directory [" +  DirName + "] -------" + self.Tool.MyNow())
        self._clear_dir(DirName)
        os.rmdir(DirName)
        self.Tool.AddMessage(u"    ... directory [" +  DirName + u"] deleted ")


    def delFileIfExists(self, fileName): # Overriding
        """"Deletes file if file exists

            Args:
                self: The reserved object 'self'
                DirName: The directory to be cleaned

        """
        if os.path.exists(fileName):
            self._force_remove_file_or_symlink(fileName)

    def BackupFiles(self, InDirName, OutDirName, D=0, Ext='*'): # Overriding
        """File archiving automation procedure

        Args:
            self: The reserved object 'self'
            InDirName: Input directory
            OutDirName: Output directory
            D: How old files to archive (number of days)
            Ext: The extension to search for ('*' - search any file)
        """
        self.Tool.AddMessage('Executing file, newer than ' +  str(D) + ' days, archiving from [' +  InDirName + '] to [' +  OutDirName + ']')
        d = date.today() - timedelta(days=D)
        d = datetime(year=d.year, month=d.month, day=d.day)
        for file_path in self.FindFileByDate(InDirName, Ext, d, 'Old'):
            self.BackupOneFile(file_path, OutDirName)

    def BackupOneFile(self, InFileName, OutDirName): # Overriding
        """Specific file archiving automation procedure

        Args:
            self: The reserved object 'self'
            InFileName: Input file
            OutDirName: Output directory
        """
        self.Tool.AddMessage("--- Archiving file [" +  InFileName + "] to [" + OutDirName + "] - " + self.Tool.MyNow())
        if os.path.exists(InFileName):
            shutil.copy(InFileName, OutDirName + '\\' + os.path.split(InFileName)[1].split(".")[-2] + "_" + self.Tool.MyNowFile() + os.path.splitext(InFileName)[1])
            self._force_remove_file_or_symlink(InFileName)
        else:
            self.Tool.AddMessage("    File " + InFileName + " does not exist ...")

    # http://stackoverflow.com/questions/1889597/deleting-directory-in-python
    def _remove_readonly(self, fn, path_, excinfo):
        """Remove read-only files and directories

        Args:
            self: The reserved object 'self'
            fn: Function for removing either a directory or a file
            path_: Path to the directory/file
        """
        if fn is os.rmdir:
            os.chmod(path_, stat.S_IWRITE)
            os.rmdir(path_)
        elif fn is os.remove:
            os.chmod(path_, stat.S_IWRITE)
            os.remove(path_)


    def _force_remove_file_or_symlink(self, path_):
        """Force remove files and symlinks

        Args:
            self: The reserved object 'self'
            path_: Path to the file/symlink
        """
        try:
            os.remove(path_)
        except OSError:
            os.chmod(path_, stat.S_IWRITE)
            os.remove(path_)


    def _is_regular_dir(self, path_):
        """Check if directory is regular directory

        Args:
            self: The reserved object 'self'
            path_: Path to the directory
        """
        try:
            mode = os.lstat(path_).st_mode
        except os.error:
            mode = 0
        return stat.S_ISDIR(mode)


    def _clear_dir(self, path_, pattern='*'):
        """Clear directory contents

        Args:
            self: The reserved object 'self'
            path_: Path to the directory
            pattern: Check if the file matches the given pattern (Default = '*'(Matches everything))
        """
        if self._is_regular_dir(path_):
            # Given path is a directory, clear its contents
            for name in [filename for filename in os.listdir(path_)  if fnmatch.fnmatch(filename, pattern)]:
                fullpath = os.path.join(path_, name)
                if self._is_regular_dir(fullpath):
                    shutil.rmtree(fullpath, onerror=self._remove_readonly)
                else:
                    self._force_remove_file_or_symlink(fullpath)
        else:
            # Given path is a file or a symlink.
            # Raise an exception here to avoid accidentally clearing the contents
            # of a symbolic linked directory.
            raise OSError("Cannot call clear_dir() on a symbolic link")



from setuptools import setup
import os
import re

def get_version():
    """Get version number from SysGISToolsSysParams file"""
    #versFile = 'GISPython\\SysGISToolsSysParams.py'
    versFile = os.path.join('GISPython', 'SysGISToolsSysParams.py') # os.path.join for Unix compatibility
    versFDir = os.path.dirname(os.path.abspath(__file__))
    versionpath = os.path.join(versFDir, versFile)
    fileRead = open(versionpath, 'rt').readlines()
    vsre = r"^Version = ['\"]([^'\"]*)['\"]"
    for line in fileRead:
        mo = re.search(vsre, line, re.M)
        if mo:
            return str(mo.group(1)[24:])

setup(
    name = 'GISPython',
    version = get_version(), # version number according to PEP440 https://www.python.org/dev/peps/pep-0440/
    description = 'Additional tools for administering and automating different ArcPy and ArcGIS Server geoprocessing operations. Package is intended for use with ArcGIS 10.2.1 and later (has been tested on ArcGIS 10.3.1, ArcGIS 10.4, ArcGIS 10.5, ArcGIS 10.6.1)', # short package description
    long_description = 'For readme see GitHub https://github.com/lvmgeo/GISPython', # if needed (entire documentation)
    url = 'https://github.com/lvmgeo/GISPython', # GitHub url
    author = 'LVM GEO', # author
    author_email = 'lvmGEOGit@lvm.lv', # email
    license = 'GPLv3', # license

    # classifiers for PyPi: https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # Project development status
        'Development Status :: 5 - Production/Stable',
        # Intended audience and topic
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Topic :: Scientific/Engineering :: GIS',
        'Topic :: Software Development :: Libraries :: Python Modules',
        # License https://choosealicense.com/
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        # Operating system
        'Operating System :: Microsoft :: Windows',

        # Specify the Python versions you support here.
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
    ],

    keywords = ['ArcGIS', 'ArcPy', 'ArcGIS Server', 'automation'], # keywords (string or list of strings)
    packages = ["GISPython"], # packages to install
    include_package_data = True,
    install_requires = ['paramiko>=2.1.2', 'simplejson>=3.10.0', 'patool>=1.12', 'python-ntlm>=1.1.0', 'lxml>=4.3.1'], # list of packages required in project (these will be installed by pip)

    # entry_points = {'console_scripts': [
        # 'GISPython = __init__:main'
        # ]
    # },
)



# -*- coding: utf-8 -*-
"""
     Module for Json parameter file procedures
"""

import os
import codecs
import simplejson as json
from collections import OrderedDict

class JsonParams(object):
    """Json parameter reading support class"""
    def __init__(self, Tool, ConfigFolder, ConfigFile):
        """Class initialization procedure

        Args:
            self: The reserved object 'self'
            Tool: GEOPython tool (optional)
            ConfigFolder: Configuration file storing directory
            ConfigFile: Name of the configuration file (without extension)
        """
        self.Tool = Tool
        self.ConfigFolder = ConfigFolder
        self.ConfigFile = ConfigFile
        if not Tool == None:
            self.ConfigPath = os.path.join(self.Tool.ExecutePatch, ConfigFolder, ConfigFile)
        else:
            if not ConfigFolder == None:
                self.ConfigPath = os.path.join(ConfigFolder, ConfigFile)
            else:
                self.ConfigPath = ConfigFile
        if not self.ConfigPath.lower()[-4:] == "json":
            self.ConfigPath = self.ConfigPath + '.json'
        self.Params = []

    def GetParams(self):
        """Get parameters from the parameter file

        Args:
            self: The reserved object 'self'
        """
        f = codecs.open(self.ConfigPath, 'r', 'utf-8')
        JsonString = f.read()
        f.close()
        J = json.loads(JsonString, object_pairs_hook=OrderedDict)
        self.Params = J
        return self.Params

    def WriteParams(self, sort_keys=True):
        """Save parameters in the parameter file

        Args:
            self: The reserved object 'self'
        """
        JsonString = json.dumps(self.Params, sort_keys, indent=4 * ' ')
        f = codecs.open(self.ConfigPath, 'w', 'utf-8')
        f.write(JsonString)
        f.close()

    def UpdateValueByPath(self, path, Value, valueIsStringJson = False):
        elem = self.Params

        if valueIsStringJson:
            Value = json.loads(Value, object_pairs_hook=OrderedDict)

        pathList = path.strip("\\").split("\\")
        for key in pathList[:-1]:
                elem = self.__get_sub_element__(elem, key)
        LastKey = pathList[-1:][0]

        elem[LastKey] = Value

    def AppendValueByPath(self, path, key, Value, valueIsStringJson = False):
        elem = self.Params

        if valueIsStringJson:
            Value = json.loads(Value, object_pairs_hook=OrderedDict)
            
        if not path == '':
            pathList = path.strip("\\").split("\\")
            for pathkey in pathList:
                    elem = self.__get_sub_element__(elem, pathkey)

        if isinstance(elem, list): 
            elem.insert(key, Value)
        else:
            elem[key] = Value

    def GetValueByPath(self, path):
        elem = self.Params
        pathList = path.strip("\\").split("\\")
        for key in pathList:
                elem = self.__get_sub_element__(elem, key)

        return elem

    def __get_sub_element__(self, element, key):
        position = 0
        if key.find('[')>-1 and key.find(']')>-1:
            position = int(key.replace(']','').replace('[',''))
            return element[position]
        else:
            return element[key]


# -*- coding: utf-8 -*-
"""
    SFTP operations module
"""

import paramiko

class SFTPHelper:
    """Class for easing the SFTP operations"""

    def __init__(self, userName, password, host, port, pkey_file = None):
        """Class initialization procedure

        Args:
            self: The reserved object 'self'
            username: Username
            password: Password (provide None if using pkey_file parameter)
            host: IP adress
            port: Port number
            pkey_file: path to rsa private key file
        """
        self.host = host
        self.port = port
        self.username = userName
        self.password = password

        if not pkey_file is None:
            pkey = paramiko.RSAKey.from_private_key_file(pkey_file)

        # Transporta objekta izveide
        self.transport = paramiko.Transport((self.host, self.port))
        if pkey_file is None:
            self.transport.connect(username=self.username, password=self.password)
        else:
            self.transport.connect(username=self.username, pkey = pkey)

        # SFTP objekta izveide
        self.sftp = paramiko.SFTPClient.from_transport(self.transport)

    def upload(self, local, remote):
        """Upload the file, using SFTP

        Args:
            self: The reserved object 'self'
            local: Uploadable file local path
            remote: Uploadable file path on the server
        """
        self.sftp.put(local, remote)

    def download(self, remote, local):
        """Download the file, using SFTP

        Args:
            self: The reserved object 'self'
            remote: Downloadable file path on the server
            local: Local download path
        """
        self.sftp.get(remote, local)

    def close(self):
        """Closes the connection, if it is active"""
        if self.transport.is_active():
            self.sftp.close()
            self.transport.close()



# -*- coding: utf-8 -*-
"""
     MXD hepler functions
"""

import TimerHelper
import GDBHelper
import os, sys, traceback

class MXDHelper:
    """Class for easing the MXD file operations"""

    def __init__(self, gp=None, Tool=None):
            """Class initialization procedure
                Args:
                    self: The reserved object 'self'
                    gp: ArcPy GP object
                    Tool: Reference to the GISTools10 object
            """
            if Tool == None:
                self.gp = gp
                self.isTool = False
            else:
                self.isTool = True
                self.gp = Tool.gp
                self.Tool = Tool
    
    def AddWarning(self, str):
        """Helper function to unify use of AddWarning with GISPython Tool or without"""
        if self.isTool:
            self.Tool.AddWarning(str)
        else:
            self.gp.AddWarning(str)
    
    def AddMessage(self, str):
        """Helper function to unify use of AddMessage with GISPython Tool or without"""
        if self.isTool:
            self.Tool.AddMessage(str)
        else:
            self.gp.AddMessage(str)

    def OutputMessages(self, ErrorSeverity=2):
        """Helper function to unify use of OutputMessages with GISPython Tool or without"""
        if self.isTool:
            self.Tool.OutputMessages(str)
        else:
            maxSeverity = self.gp.GetMaxSeverity()
            if maxSeverity > 1:
                self.gp.AddMessage('')
                self.gp.AddMessage('--------------------------------------------')
                for i in range(0, self.gp.GetMessageCount()):
                    self.gp.AddReturnMessage(i)
                self.gp.AddMessage('--------------------------------------------')

    def ChangeMXDSource(self, mxd, conn, difOutDir = False, outDir = "#", silent=False):
        """Changes MXD file connection and saves the file in user specified directory
            Args:
                self: The reserved object 'self'
                mxd: path to MXD,
                conn: new connection,
                difOutDir: is the output dir diferent from source (do the MXD be saved in thesame path ('TRUE'))
                outDir: directory to with to copy new MXD,
                silent: does the messages will be supressed"""

        T = TimerHelper.TimerHelper()
        if not silent == True:
            self.AddMessage(u'    Processing document [{0}]'.format(mxd))
        if not self.gp.Exists(mxd):
            self.AddWarning(u'MXD with path {0} has not been found'.format(MXD))
        else:
            if not silent == True:
                 self.AddMessage(u'        Document found')
            T.TimerReset()
            try:
                Doc = self.gp.mapping.MapDocument(mxd)
                if not silent == True: 
                    self.AddMessage(u'        Document opened - {0}'.format(T.GetTimeReset()))
                try:
                    Doc.replaceWorkspaces('', 'NONE', conn, 'SDE_WORKSPACE', True)
                    #Doc.findAndReplaceWorkspacePaths("", conn, True)
                    if not silent == True: 
                        self.AddMessage(u'        Document sources changed - {0}'.format(T.GetTimeReset()))
                    try:
                        if difOutDir == True:
                            sourceDir = os.path.basename(mxd)
                            outPath = mxd.replace(sourceDir, outDir)
                            outDir = os.path.dirname(outPath)
                            if not os.path.exists(outDir):
                                os.makedirs(outDir)
                            Doc.saveACopy(outPath)
                        else:
                            Doc.save()
                        del Doc
                        if not silent == True: 
                            self.AddMessage(u'        Document saved - {0}'.format(T.GetTimeReset()))
                    except:
                        self.AddWarning(u'MXD with path {0} cannot be saved'.format(mxd))
                        tb = sys.exc_info()
                        self.AddWarning("Line %i" % tb[2].tb_lineno)
                        self.AddWarning(traceback.format_exc())
                except:
                    self.AddWarning(u'MXD sources with path {0} cannot be replaced'.format(mxd))
                    tb = sys.exc_info()
                    self.AddWarning("Line %i" % tb[2].tb_lineno)
                    self.AddWarning(traceback.format_exc())
            except:
                self.AddWarning(u'MXD with path {0} cannot be opened'.format(mxd))
                tb = sys.exc_info()
                self.AddWarning("Line %i" % tb[2].tb_lineno)
                self.AddWarning(traceback.format_exc())

    def AddRealatedTablesToMXD(self, MXD, silent=False):
        """
            Procedure adds all mising related tables to MXD
            Args:
                self: The reserved object 'self'
                mxd: path to MXD,
                silent: does the messages will be supressed
        """
        
        self.GDBTools = GDBHelper.GDBHelper(self.gp)
        T = TimerHelper.TimerHelper()
        if not silent == True:
            self.AddMessage(u'    Processing document [{0}]'.format(MXD))
        if not self.gp.Exists(MXD):
            if not silent == True:
                self.AddWarning(u'MXD with path {0} has not been found'.format(MXD))
        else:
            if not silent == True:
                 self.AddMessage(u'        Document found')
            T.TimerReset()
            try:
                Doc = self.gp.mapping.MapDocument(MXD)
                if not silent == True: 
                    self.AddMessage(u'        Document opened - {0}'.format(T.GetTimeReset()))
                try:
                    Relations = list()
                    for lyr in self.gp.mapping.ListLayers(Doc):
                        self.__ProcessLyr(lyr,'            ', Relations, silent = silent)
                    Relations = set(Relations)

                    if not silent == True: 
                        self.AddMessage('\n           ' + u'Found tables:')
                        for tabl in Relations:
                            self.AddMessage('              > ' + tabl)

                    remList = list()
                    for tab in self.gp.mapping.ListTableViews(Doc):
                        for rel in Relations:
                            if tab.datasetName.lower() == rel.lower().split('\\')[-1]:
                                remList.append(rel)
                    Relations = [x for x in Relations if x not in remList]

                    if not silent == True: 
                        self.AddMessage('\n           ' + u'Tables missing from MXD:')
                        for tabl in Relations:
                            self.AddMessage('              > ' + tabl)
                    
                    for rel in Relations:
                        addTable = self.gp.mapping.TableView(rel)
                        self.gp.mapping.AddTableView(Doc.activeDataFrame, addTable)

                    if not silent == True: 
                        self.AddMessage(u'\n        Document tables processed - {0}'.format(T.GetTimeReset()))
                    try:
                        Doc.save()
                        del Doc
                        if not silent == True: 
                            self.AddMessage(u'        Document saved - {0}'.format(T.GetTimeReset()))
                    except:
                        self.AddWarning(u'MXD with path {0} cannot be saved'.format(MXD))
                        tb = sys.exc_info()
                        self.AddWarning("Line %i" % tb[2].tb_lineno)
                        self.AddWarning(traceback.format_exc())
                except:
                    self.AddWarning(u'MXD sources with path {0} cannot be processed'.format(MXD))
                    tb = sys.exc_info()
                    self.AddWarning("Line %i" % tb[2].tb_lineno)
                    self.AddWarning(traceback.format_exc())
            except:
                self.AddWarning(u'MXD with path {0} cannot be opened'.format(MXD))
                tb = sys.exc_info()
                self.AddWarning("Line %i" % tb[2].tb_lineno)
                self.AddWarning(traceback.format_exc())

    def __ProcessLyr(self, lyr, step, Tables, silent=False):
        if lyr.isFeatureLayer == True:
            if not silent == True: 
                self.AddMessage(step + lyr.name)
            Relations = list()
            Relations = self.GDBTools.GetRelations(lyr.workspacePath, self.gp.Describe(lyr.dataSource), Relations)
            for rel in Relations:
                DRel = self.gp.Describe(os.path.join(lyr.workspacePath, rel))
                if DRel.dataElementType == u'DETable':
                    Tables.append(os.path.join(lyr.workspacePath, rel).lower())
                    if not silent == True: 
                        self.AddMessage(step + '    ' + lyr.name + " -> " + rel)
        if lyr.isGroupLayer == True:
            if not silent == True: 
                self.AddMessage(step + lyr.name + ':')
            for sublyr in self.gp.mapping.ListLayers(lyr):
                if not silent == True: 
                    self.__ProcessLyr(sublyr, step + '    ', Tables)


# -*- coding: utf-8 -*-
"""
    GIS function support module
"""
import GDBHelper
import GISPythonToolBase
import codecs

class GISTools10(GISPythonToolBase.GISPythonToolBase):
    """Class for storing the auxiliary batch processing and GIS geoprocesing functions"""
    def __init__(self, ToolName, Params, licenceLevel = "arceditor"):
        """Class initialization procedure

        Args:
            self: The reserved object 'self'
            ToolName: Name of the tool (used for output)
            Params: parameter object
            licenceLevel: arcinfo, arceditor, arcview, arcserver, arcenginegeodb, or arcengine
        """
        super(GISTools10, self).__init__(ToolName, Params)

        if licenceLevel == "arcinfo":
            print(u'...Using licence level : ' + licenceLevel)
            import arcinfo
        elif licenceLevel == "arceditor":
            print(u'...Using licence level : ' + licenceLevel)
            import arceditor
        elif licenceLevel == "arcview":
            print(u'...Using licence level : ' + licenceLevel)
            import arcview
        elif licenceLevel == "arcserver":
            print(u'...Using licence level : ' + licenceLevel)
            import arcserver
        elif licenceLevel == "arcenginegeodb":
            print(u'...Using licence level : ' + licenceLevel)
            import arcenginegeodb
        elif licenceLevel == "arcengine":
            print(u'...Using licence level : ' + licenceLevel)
            import arcengine
        else:
            print(u'...Incorect licence suplied - using : arceditor')
            import arceditor
            
        import arcpy # arcpy variables
        self.gp = arcpy
        if hasattr(Params, 'SetLogHistory'):
            arcpy.SetLogHistory(Params.SetLogHistory)
        else:
            arcpy.SetLogHistory(False)
        self.GDBHelper = GDBHelper.GDBHelper(self.gp)
        self.SR = MySR()

        # Set up Logging
        LogDir = self.Pr.ErrorLogDir
        LogDirArh = self.Pr.ErrorLogDirArh
        OutDir = self.Pr.OutDir
        OutDirArh = self.Pr.OutDirArh

        try:
            self.AchiveFiles(LogDir, LogDirArh, ToolName, False)
        except Exception, err:
            print(u'Error archiving errlog files')
            if hasattr(err, 'strerror'):
                if hasattr(err, 'strerror'):
                    print(err.strerror)
                else:
                    print('{}'.format(err))
            else:
                print(err.message)
        try:
            self.AchiveFiles(OutDir, OutDirArh, ToolName, False)
        except Exception, err:
            print(u'Error archiving outlog files')
            if hasattr(err, 'strerror'):
                if hasattr(err, 'strerror'):
                    print(err.strerror)
                else:
                    print('{}'.format(err))
            else:
                print(err.message)
        self.fLog = codecs.open(LogDir + '\\' + self.ToolName + self.MyNowFile() + '.errlog', encoding='utf-8', mode='w')
        self.fOut = codecs.open(OutDir + '\\' + self.ToolName + self.MyNowFile() + '.outlog', encoding='utf-8', mode='w')
        self.fSQL = OutDir + '\\' + self.ToolName + self.MyNowFile() + 'SQL.outlog'

        self.AddMessage(u'\n==================================================')
        self.AddMessage(r'//////////////////////////////////////////////////')
        self.AddMessage(u'Executing the tool ' + ToolName + u' ' + self.MyNow())
        self.AddMessage(r'\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\')
        self.AddMessage(u'==================================================\n')

    def callGP(self, functionName, *args):
        """Function to call the arcPy GP functions with automatic output messge display and output result returning

        Args:
            self: The reserved object 'self'
            functionName: Name of the arcPy GP function
            args: arguments as Tuple
        """
        gpOutput = self.callGPSilent(functionName, *args)
        self.OutputMessages()
        return gpOutput

    def callGPSilent(self, functionName, *args):
        """Function to call the arcPy GP functions without output

        Args:
            self: The reserved object 'self'
            functionName: Name of the arcPy GP function
            args: arguments as Tuple
        """
        gpFunction = getattr(self.gp, functionName)
        rezult = gpFunction(*args)
        if rezult.outputCount > 0:
            gpOutput = rezult.getOutput(0)
        else:
            gpOutput = None
        return gpOutput

    def MyDispose(self):
        """Disposal of the class

        Args:
            self: The reserved object 'self'
        """
        super(GISTools10, self).MyDispose()
        self.fOut.close()
        self.fLog.close()

    def OutputMessages(self, ErrorSeverity=2):
        """Procedure to output messages stored in the GP object

        Args:
            self: The reserved object 'self'
            ErrorSeverity: Maximum Severity to report as error
        """
        self.AddMessage('\n--------------------------------------------')
        maxSeverity = self.gp.GetMaxSeverity()
        for i in range(0, self.gp.GetMessageCount()):
            self.gp.AddReturnMessage(i)
            #self.gp.AddMessage(self.gp.GetMessage(i))
            self.fOut.write(self.gp.GetMessage(i))
            self.OutputStr += self.gp.GetMessage(i)
            self.fOut.write('\n')
            self.OutputStr += '\n'
            self.fOut.flush()
            if maxSeverity >= ErrorSeverity:
                self.fLog.write(self.gp.GetMessage(i))
                self.OutputErrStr += self.gp.GetMessage(i)
                self.fLog.write('\n')
                self.OutputErrStr += '\n'
                self.fLog.flush()
        self.AddMessage('--------------------------------------------')

    def OutputErrors(self):
        """Procedure to output messages and errors stored in the GP object.
        !!! Depricated - Left for backwards compatibility - use OutputMessages with ErrorSeverity 0 !!!

        Args:
            self: The reserved object 'self'
        """
        self.AddMessage('')
        self.AddMessage('--------------------------------------------')
        for i in range(0, self.gp.GetMessageCount()):
            self.gp.AddReturnMessage(i)
            self.fOut.write(self.gp.GetMessage(i))
            self.OutputStr += self.gp.GetMessage(i)
            self.fOut.write('\n')
            self.OutputStr += '\n'
            self.fOut.flush()
            self.fLog.write(self.gp.GetMessage(i))
            self.OutputErrStr += self.gp.GetMessage(i)
            self.fLog.write('\n')
            self.OutputErrStr += '\n'
            self.fLog.flush()
        self.AddMessage('--------------------------------------------')

    def AddMessage(self, strMessage, newline=True):
        """Procedure for a message output (screen, logfile and if necessary e-mail)

        Args:
            self: The reserved object 'self'
            strMessage: Output message text
            newline: Flag that marks that the output must be continued in a new line
        """
        if strMessage == None:
            strMessage = ''
        if not isinstance(strMessage, unicode):
            if not isinstance(strMessage, str):
                strMessage = str(strMessage)
        if strMessage[-1:] == '\n':
            strMessage = strMessage[0:-1]
        try:
            self.fOut.write(strMessage)
            self.OutputStr += strMessage
        except:
            self.fOut.write(self._tryCovertStringEncoding(strMessage))
            self.OutputStr += self._tryCovertStringEncoding(strMessage)

        try:
            self.gp.AddMessage(strMessage)
        except:
            print strMessage
        if newline == True:
            self.fOut.write('\n')
            self.OutputStr += '\n'
        self.fOut.flush()

    def AddError(self, strMessage, newline=True):
        """Procedure for the GP object error message output (screen, logfile and if necessary e-mail)

        Args:
            self: The reserved object 'self'
            strMessage: Output message text
        """
        if strMessage == None:
            strMessage = ''
        if not isinstance(strMessage, unicode):
            if not isinstance(strMessage, str):
                strMessage = str(strMessage)
        if strMessage[-1:] == '\n':
            strMessage = strMessage[0:-1]
        try:
            self.gp.AddError(strMessage)
        except:
            print strMessage
        try:
            self.fLog.write(strMessage)
            self.OutputErrStr += strMessage
        except:
            self.fLog.write(self._tryCovertStringEncoding(strMessage))
            self.OutputErrStr += self._tryCovertStringEncoding(strMessage)
        if newline == True:
            self.fLog.write('\n')
            self.OutputErrStr += '\n'
        self.fLog.flush()
        try:
            self.fOut.write(strMessage)
            self.OutputStr += strMessage
        except:
            self.fOut.write(self._tryCovertStringEncoding(strMessage))
            self.OutputStr += self._tryCovertStringEncoding(strMessage)
        if newline == True:
            self.fOut.write('\n')
            self.OutputStr += '\n'
        self.fOut.flush()

    def AddWarning(self, strMessage):
        """Procedure for the GP object warning message output (screen, logfile and if necessary e-mail)

        Args:
            self: The reserved object 'self'
            strMessage: Output message text
        """
        if strMessage == None:
            strMessage = ''
        if not isinstance(strMessage, unicode):
            if not isinstance(strMessage, str):
                strMessage = str(strMessage)
        if strMessage[-1:] == '\n':
            strMessage = strMessage[0:-1]
        try:
            self.gp.AddWarning(strMessage)
        except:
            print strMessage
        try:
            self.fLog.write(strMessage)
            self.OutputErrStr += strMessage
        except:
            self.fLog.write(self._tryCovertStringEncoding(strMessage))
            self.OutputErrStr += self._tryCovertStringEncoding(strMessage)
        self.fLog.write('\n')
        self.OutputErrStr += '\n'
        self.fLog.flush()
        try:
            self.fOut.write(strMessage)
            self.OutputStr += strMessage
        except:
            self.fOut.write(self._tryCovertStringEncoding(strMessage))
            self.OutputStr += self._tryCovertStringEncoding(strMessage)
        self.fOut.write('\n')
        self.OutputStr += '\n'
        self.fOut.flush()



class MySR():
    """Class for storing often used coordinate system parameters"""
    def __init__(self):
        """Class initialization procedure

        Args:
            self: The reserved object 'self'
        """
        self.LKS92_0 = "PROJCS['LKS_1992_Latvia_TM_0',GEOGCS['GCS_LKS_1992',DATUM['D_Latvia_1992',SPHEROID['GRS_1980',6378137.0,298.257222101]],PRIMEM['Greenwich',0.0],UNIT['Degree',0.0174532925199433]],PROJECTION['Transverse_Mercator'],PARAMETER['False_Easting',500000.0],PARAMETER['False_Northing',0.0],PARAMETER['Central_Meridian',24.0],PARAMETER['Scale_Factor',0.9996],PARAMETER['Latitude_Of_Origin',0.0],UNIT['Meter',1.0]]"
        self.LKS92 = "PROJCS['LKS_1992_Latvia_TM',GEOGCS['GCS_LKS_1992',DATUM['D_Latvia_1992',SPHEROID['GRS_1980',6378137.0,298.257222101]],PRIMEM['Greenwich',0.0],UNIT['Degree',0.0174532925199433]],PROJECTION['Transverse_Mercator'],PARAMETER['False_Easting',500000.0],PARAMETER['False_Northing',-6000000.0],PARAMETER['Central_Meridian',24.0],PARAMETER['Scale_Factor',0.9996],PARAMETER['Latitude_Of_Origin',0.0],UNIT['Meter',1.0]]"
        self.WebMercator = "PROJCS['WGS_1984_Web_Mercator_Auxiliary_Sphere',GEOGCS['GCS_WGS_1984',DATUM['D_WGS_1984',SPHEROID['WGS_1984',6378137.0,298.257223563]],PRIMEM['Greenwich',0.0],UNIT['Degree',0.0174532925199433]],PROJECTION['Mercator_Auxiliary_Sphere'],PARAMETER['False_Easting',0.0],PARAMETER['False_Northing',0.0],PARAMETER['Central_Meridian',0.0],PARAMETER['Standard_Parallel_1',0.0],PARAMETER['Auxiliary_Sphere_Type',0.0],UNIT['Meter',1.0]]"
        self.WGS84 = "GEOGCS['GCS_WGS_1984',DATUM['D_WGS_1984',SPHEROID['WGS_1984',6378137.0,298.257223563]],PRIMEM['Greenwich',0.0],UNIT['Degree',0.0174532925199433]]"
